@@ -131,6 +131,25 @@ def _get_type_for_expr(expr: Expression, api: SemanticAnalyzerPluginInterface):
     raise ValueError(f"Unsupported expression {type(expr)}")
 
 
+def create_type_hook(ctx: DynamicClassDefContext) -> None:
+    # returning classes/type aliases is not supported yet by mypy
+    # see https://github.com/python/mypy/issues/5865
+
+    type_alias = TypeAlias(
+        AnyType(TypeOfAny.from_error),
+        fullname=ctx.api.qualified_name(ctx.name),
+        line=ctx.call.line,
+        column=ctx.call.column,
+    )
+
+    ctx.api.add_symbol_table_node(
+        ctx.name,
+        SymbolTableNode(GDEF, type_alias, plugin_generated=True),
+    )
+
+    return
+
+
 def union_hook(ctx: DynamicClassDefContext) -> None:
     try:
         # Check if types is passed as a keyword argument
@@ -602,6 +621,9 @@ class StrawberryPlugin(Plugin):
         if self._is_strawberry_enum(fullname):
             return enum_hook
 
+        if self._is_strawberry_create_type(fullname):
+            return create_type_hook
+
         return None
 
     def get_function_hook(
@@ -691,6 +713,17 @@ class StrawberryPlugin(Plugin):
             }
         )
 
+    def _is_strawberry_create_type(self, fullname: str) -> bool:
+        # using endswith(.create_type) is not ideal as there might be
+        # other function called like that, but it's the best we can do
+        # when follow-imports is set to "skip". Hopefully in the future
+        # we can remove our custom hook for create type
+
+        return (
+            fullname == "strawberry.tools.create_type.create_type"
+            or fullname.endswith(".create_type")
+        )
+
     def _is_strawberry_pydantic_decorator(self, fullname: str) -> bool:
         if any(
             strawberry_decorator in fullname
