@@ -1,6 +1,6 @@
 from asyncio import ensure_future
 from inspect import isawaitable
-from typing import Any, Awaitable, List, Optional, Sequence, Type, cast
+from typing import Any, Awaitable, Collection, List, Optional, Sequence, Type, cast
 
 from graphql import (
     ExecutionContext as GraphQLExecutionContext,
@@ -10,7 +10,7 @@ from graphql import (
     execute as original_execute,
     parse,
 )
-from graphql.validation import validate
+from graphql.validation import ValidationRule, validate
 
 from strawberry.extensions import Extension
 from strawberry.extensions.runner import ExtensionsRunner
@@ -25,6 +25,7 @@ async def execute(
     additional_middlewares: List[Any] = None,
     execution_context_class: Optional[Type[GraphQLExecutionContext]] = None,
     validate_queries: bool = True,
+    validation_rules: Optional[Collection[Type[ValidationRule]]] = None,
 ) -> ExecutionResult:
     extensions_runner = ExtensionsRunner(
         execution_context=execution_context,
@@ -35,12 +36,12 @@ async def execute(
 
     additional_middlewares = additional_middlewares or []
 
-    with extensions_runner.request():
+    async with extensions_runner.request():
         # Note: In graphql-core the schema would be validated here but in
         # Strawberry we are validating it at initialisation time instead
 
         try:
-            with extensions_runner.parsing():
+            async with extensions_runner.parsing():
                 document = parse(query)
                 execution_context.graphql_document = document
         except GraphQLError as error:
@@ -48,7 +49,7 @@ async def execute(
             return ExecutionResult(
                 data=None,
                 errors=[error],
-                extensions=extensions_runner.get_extensions_results(),
+                extensions=await extensions_runner.get_extensions_results(),
             )
 
         except Exception as error:  # pragma: no cover
@@ -58,12 +59,12 @@ async def execute(
             return ExecutionResult(
                 data=None,
                 errors=[error],
-                extensions=extensions_runner.get_extensions_results(),
+                extensions=await extensions_runner.get_extensions_results(),
             )
 
         if validate_queries:
-            with extensions_runner.validation():
-                validation_errors = validate(schema, document)
+            async with extensions_runner.validation():
+                validation_errors = validate(schema, document, rules=validation_rules)
 
             if validation_errors:
                 execution_context.errors = validation_errors
@@ -90,7 +91,7 @@ async def execute(
     return ExecutionResult(
         data=result.data,
         errors=result.errors,
-        extensions=extensions_runner.get_extensions_results(),
+        extensions=await extensions_runner.get_extensions_results(),
     )
 
 
@@ -102,6 +103,7 @@ def execute_sync(
     additional_middlewares: List[Any] = None,
     execution_context_class: Optional[Type[GraphQLExecutionContext]] = None,
     validate_queries: bool = True,
+    validation_rules: Optional[Collection[Type[ValidationRule]]] = None,
 ) -> ExecutionResult:
     extensions_runner = ExtensionsRunner(
         execution_context=execution_context,
@@ -125,7 +127,7 @@ def execute_sync(
             return ExecutionResult(
                 data=None,
                 errors=[error],
-                extensions=extensions_runner.get_extensions_results(),
+                extensions=extensions_runner.get_extensions_results_sync(),
             )
 
         except Exception as error:  # pragma: no cover
@@ -135,12 +137,12 @@ def execute_sync(
             return ExecutionResult(
                 data=None,
                 errors=[error],
-                extensions=extensions_runner.get_extensions_results(),
+                extensions=extensions_runner.get_extensions_results_sync(),
             )
 
         if validate_queries:
             with extensions_runner.validation():
-                validation_errors = validate(schema, document)
+                validation_errors = validate(schema, document, rules=validation_rules)
 
             if validation_errors:
                 execution_context.errors = validation_errors
@@ -169,5 +171,5 @@ def execute_sync(
     return ExecutionResult(
         data=result.data,
         errors=result.errors,
-        extensions=extensions_runner.get_extensions_results(),
+        extensions=extensions_runner.get_extensions_results_sync(),
     )
