@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 from enum import Enum
-from inspect import isasyncgen, iscoroutine
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple, Type, Union
 
 
@@ -44,6 +43,7 @@ from strawberry.type import StrawberryList, StrawberryOptional, StrawberryType
 from strawberry.types.info import Info
 from strawberry.types.types import TypeDefinition
 from strawberry.union import StrawberryUnion
+from strawberry.utils.await_maybe import await_maybe
 
 from .types.concrete_type import ConcreteType
 from .types.scalar import get_scalar_type
@@ -221,11 +221,27 @@ class GraphQLCoreConverter:
 
             return graphql_fields
 
+        def resolve_type(
+            obj: Any,
+            info: GraphQLResolveInfo,
+            type_: Union[GraphQLInterfaceType, GraphQLUnionType],
+        ) -> GraphQLObjectType:
+            # TODO: this will probably break when passing dicts
+            # or even non strawberry types
+            resolved_type = self.type_map[
+                obj.__class__._type_definition.name
+            ].implementation
+
+            assert isinstance(resolved_type, GraphQLObjectType)
+
+            return resolved_type
+
         graphql_interface = GraphQLInterfaceType(
             name=interface.name,
             fields=get_graphql_fields,
             interfaces=list(map(self.from_interface, interface.interfaces)),
             description=interface.description,
+            resolve_type=resolve_type,
         )
 
         self.type_map[interface.name] = ConcreteType(
@@ -260,14 +276,6 @@ class GraphQLCoreConverter:
             assert isinstance(graphql_object_type, GraphQLObjectType)  # For mypy
             return graphql_object_type
 
-        # Only define an is_type_of function for Types that implement an interface.
-        # Otherwise, leave it to the default implementation
-        is_type_of = (
-            (lambda obj, _: isinstance(obj, object_type.origin))
-            if object_type.interfaces
-            else None
-        )
-
         def get_graphql_fields() -> Dict[str, GraphQLField]:
             graphql_fields = {}
 
@@ -282,7 +290,6 @@ class GraphQLCoreConverter:
             name=object_type.name,
             fields=get_graphql_fields,
             interfaces=list(map(self.from_interface, object_type.interfaces)),
-            is_type_of=is_type_of,
             description=object_type.description,
         )
 
@@ -336,49 +343,54 @@ class GraphQLCoreConverter:
                     message = getattr(permission, "message", None)
                     raise PermissionError(message)
 
+        async def _check_permissions_async(
+            source: Any, info: Info, kwargs: Dict[str, Any]
+        ):
+            for permission_class in field.permission_classes:
+                permission = permission_class()
+                has_permission: bool
+
+                has_permission = await await_maybe(
+                    permission.has_permission(source, info, **kwargs)
+                )
+
+                if not has_permission:
+                    message = getattr(permission, "message", None)
+                    raise PermissionError(message)
+
         def _strawberry_info_from_graphql(info: GraphQLResolveInfo) -> Info:
             return Info(
-                field_name=info.field_name,
-                field_nodes=info.field_nodes,
-                context=info.context,
-                root_value=info.root_value,
-                variable_values=info.variable_values,
-                return_type=field.type,
-                operation=info.operation,
-                path=info.path,
+                _raw_info=info,
+                _field=field,
+            )
+
+        def _get_result(_source: Any, info: Info, **kwargs):
+            field_args, field_kwargs = _get_arguments(
+                source=_source, info=info, kwargs=kwargs
+            )
+
+            return field.get_result(
+                _source, info=info, args=field_args, kwargs=field_kwargs
             )
 
         def _resolver(_source: Any, info: GraphQLResolveInfo, **kwargs):
             strawberry_info = _strawberry_info_from_graphql(info)
             _check_permissions(_source, strawberry_info, kwargs)
 
-            args, kwargs = _get_arguments(
-                source=_source, info=strawberry_info, kwargs=kwargs
-            )
+            return _get_result(_source, strawberry_info, **kwargs)
 
-            result = field.get_result(
-                _source, info=strawberry_info, args=args, kwargs=kwargs
-            )
+        async def _async_resolver(_source: Any, info: GraphQLResolveInfo, **kwargs):
+            strawberry_info = _strawberry_info_from_graphql(info)
+            await _check_permissions_async(_source, strawberry_info, kwargs)
 
-            if isasyncgen(result):
+            return await await_maybe(_get_result(_source, strawberry_info, **kwargs))
 
-                async def yield_results(results):
-                    async for value in results:
-                        yield value
-
-                return yield_results(result)
-
-            if iscoroutine(result):  # pragma: no cover
-
-                async def await_result(result):
-                    return await result
-
-                return await_result(result)
-
-            return result
-
-        _resolver._is_default = not field.base_resolver  # type: ignore
-        return _resolver
+        if field.is_async:
+            _async_resolver._is_default = not field.base_resolver  # type: ignore
+            return _async_resolver
+        else:
+            _resolver._is_default = not field.base_resolver  # type: ignore
+            return _resolver
 
     def from_scalar(self, scalar: Type) -> GraphQLScalarType:
         return get_scalar_type(scalar, self.type_map)
